@@ -15,7 +15,7 @@ use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 use Image::ExifTool::Exif;
 
-$VERSION = '1.02';
+$VERSION = '1.05';
 
 sub ProcessMetadata($$$);
 sub ProcessContentDescription($$$);
@@ -634,7 +634,7 @@ sub ProcessContentDescription($$$)
         my $len = shift @len;
         next unless $len;
         return 0 if $pos + $len > $dirLen;
-        my $val = $exifTool->Unicode2Byte(substr($$dataPt,$pos,$len),'II');
+        my $val = $exifTool->Unicode2Charset(substr($$dataPt,$pos,$len),'II');
         $exifTool->HandleTag($tagTablePtr, $tag, $val);
         $pos += $len;
     }
@@ -680,7 +680,7 @@ sub ReadASF($$$$$)
     my ($exifTool, $dataPt, $pos, $format, $size) = @_;
     my @vals;
     if ($format == 0) { # unicode string
-        $vals[0] = $exifTool->Unicode2Byte(substr($$dataPt,$pos,$size),'II');
+        $vals[0] = $exifTool->Unicode2Charset(substr($$dataPt,$pos,$size),'II');
     } elsif ($format == 2) { # 4-byte boolean
         @vals = ReadValue($dataPt, $pos, 'int32u', undef, $size);
         foreach (@vals) {
@@ -710,7 +710,7 @@ sub ProcessExtendedContentDescription($$$)
     my $dirLen = $$dirInfo{DirLen};
     return 0 if $dirLen < 2;
     my $count = Get16u($dataPt, 0);
-    $verbose and $exifTool->VerboseDir($$dirInfo{DirName}, $count);
+    $exifTool->VerboseDir($dirInfo, $count);
     my $pos = 2;
     my $i;
     for ($i=0; $i<$count; ++$i) {
@@ -773,7 +773,7 @@ sub ProcessCodecList($$$)
     my $dirLen = $$dirInfo{DirLen};
     return 0 if $dirLen < 20;
     my $count = Get32u($dataPt, 16);
-    $verbose and $exifTool->VerboseDir($$dirInfo{DirName}, $count);
+    $exifTool->VerboseDir($dirInfo, $count);
     my $pos = 20;
     my $i;
     my %codecType = ( 1 => 'Video', 2 => 'Audio' );
@@ -784,12 +784,12 @@ sub ProcessCodecList($$$)
         my $nameLen = Get16u($dataPt, $pos + 2) * 2;
         $pos += 4;
         return 0 if $pos + $nameLen + 2 > $dirLen;
-        my $name = Image::ExifTool::Unicode2Latin(substr($$dataPt,$pos,$nameLen),'v');
+        my $name = $exifTool->Unicode2Charset(substr($$dataPt,$pos,$nameLen),'II');
         $exifTool->HandleTag($tagTablePtr, "${type}Name", $name);
         my $descLen = Get16u($dataPt, $pos + $nameLen) * 2;
         $pos += $nameLen + 2;
         return 0 if $pos + $descLen + 2 > $dirLen;
-        my $desc = Image::ExifTool::Unicode2Latin(substr($$dataPt,$pos,$descLen),'v');
+        my $desc = $exifTool->Unicode2Charset(substr($$dataPt,$pos,$descLen),'II');
         $exifTool->HandleTag($tagTablePtr, "${type}Description", $desc);
         my $infoLen = Get16u($dataPt, $pos + $descLen);
         $pos += $descLen + 2 + $infoLen;
@@ -809,7 +809,7 @@ sub ProcessMetadata($$$)
     my $dirLen = $$dirInfo{DirLen};
     return 0 if $dirLen < 2;
     my $count = Get16u($dataPt, 0);
-    $verbose and $exifTool->VerboseDir($$dirInfo{DirName}, $count);
+    $exifTool->VerboseDir($dirInfo, $count);
     my $pos = 2;
     my $i;
     for ($i=0; $i<$count; ++$i) {
@@ -862,7 +862,11 @@ sub ProcessASF($$;$)
             $rtnVal = 1;
         }
         my $size = Image::ExifTool::Get64u(\$buff, 16) - 24;
-        if ($size > 0xffffffff) {
+        if ($size < 0) {
+            $err = 'Invalid ASF object size';
+            last;
+        }
+        if ($size > 0x7fffffff) {
             $err = 'Large ASF objects not supported';
             last;
         }
@@ -955,7 +959,7 @@ Windows Media Audio (WMA) and Windows Media Video (WMV) files.
 
 =head1 AUTHOR
 
-Copyright 2003-2006, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2007, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
