@@ -24,22 +24,24 @@ BEGIN {
 use vars qw($VERSION @ISA);
 use Image::ExifTool qw(:Utils :Vars);
 use Image::ExifTool::Shortcuts;
-use Image::ExifTool::XMP qw(EscapeHTML);
+use Image::ExifTool::HTML qw(EscapeHTML);
 use Image::ExifTool::IPTC;
+use Image::ExifTool::Canon;
 use Image::ExifTool::Nikon;
-use Image::ExifTool::Real;
 
-$VERSION = '1.46';
+$VERSION = '1.59';
 @ISA = qw(Exporter);
 
 sub NumbersFirst;
 
 # colors for html pages
-my $noteFont = '<font color="#666666">';
-my $noteFontSmall = '<font color="#666666" size="-1">';
-my $bgHeading = q{bgcolor='#ffbb77'};
-my $bgBody = q{bgcolor='#ffffff'};
-my $bgRow = q{bgcolor='#ffeebb'};
+my $noteFont = "<span class=n>";
+my $noteFontSmall = "<span class='n s'>";
+
+my $docType = q{<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"
+        "http://www.w3.org/TR/html4/loose.dtd">
+};
+
 
 my $caseInsensitive;    # flag to ignore case when sorting tag names
 
@@ -64,11 +66,11 @@ specific meta information that is extracted or written in an image.
 The tables listed below give the names of all tags recognized by ExifTool.
 },
     ExifTool2 => q{
-A B<Tag ID> or B<Index> is given in the first column of each table.  A
-B<Tag ID> is the computer-readable equivalent of a tag name, and is the
-identifier that is actually stored in the file.  An B<Index> refers to the
-location of the information, and is used if the information is stored at a
-fixed position in a data block.
+B<Tag ID>, B<Index> or B<Sequence> is given in the first column of each
+table.  A B<Tag ID> is the computer-readable equivalent of a tag name, and
+is the identifier that is actually stored in the file.  An B<Index> refers
+to the location of a value when found at a fixed position within a data
+block, and B<Sequence> gives the order of values for a serial data stream.
 
 A B<Tag Name> is the handle by which the information is accessed in
 ExifTool.  In some instances, more than one name may correspond to a single
@@ -78,11 +80,11 @@ question mark after a tag name indicates that the information is either not
 understood, not verified, or not very useful -- these tags are not extracted
 by ExifTool unless the Unknown (-u) option is enabled.  Be aware that some
 tag names are different than the descriptions printed out by default when
-extracting information with "exiftool".  To see the tag names instead of the
-descriptions, use "exiftool -s".
+extracting information with exiftool.  To see the tag names instead of the
+descriptions, use C<exiftool -s>.
 
 The B<Writable> column indicates whether the tag is writable by ExifTool.
-Anything but an "N" in this column means the tag is writable.  A "Y"
+Anything but an C<N> in this column means the tag is writable.  A C<Y>
 indicates writable information that is either unformatted or written using
 the existing format.  Other expressions give details about the information
 format, and vary depending on the general type of information.  The format
@@ -110,13 +112,13 @@ B<Note>: If you are familiar with common meta-information tag names, you may
 find that some ExifTool tag names are different than expected.  The usual
 reason for this is to make the tag names more consistent across different
 types of meta information.  To determine a tag name, either consult this
-documentation or run "exiftool -s" on a file containing the information in
+documentation or run C<exiftool -s> on a file containing the information in
 question.
 },
     EXIF => q{
 EXIF stands for "Exchangeable Image File Format".  This type of information
 is formatted according to the TIFF specification, and may be found in JPG,
-TIFF, PNG, MIFF and DNG images.
+TIFF, PNG, MIFF and WDP images, as well as many TIFF-based RAW images.
 
 The EXIF meta information is organized into different Image File Directories
 (IFD's) within an image.  The names of these IFD's correspond to the
@@ -131,17 +133,25 @@ images.
 These GPS tags are part of the EXIF standard, and are stored in a separate
 IFD within the EXIF information.
 
-ExifTool is very flexible about the input format for lat/long coordinates,
-and will accept 3 floating point numbers (for degrees, minutes and seconds)
-separated by just about anything.  Many other GPS tags have values which are
-fixed-length strings.  For these, the indicated string lengths include a
-null terminator which is added automatically by ExifTool.
+ExifTool is very flexible about the input format when writing lat/long
+coordinates, and will accept from 1 to 3 floating point numbers (for decimal
+degrees, degrees and minutes, or degrees, minutes and seconds) separated by
+just about anything, and will format them properly according to the EXIF
+specification.
+
+Some GPS tags have values which are fixed-length strings. For these, the
+indicated string lengths include a null terminator which is added
+automatically by ExifTool.  Remember that the descriptive values are used
+when writing (ie. 'Above Sea Level', not '0') unless the print conversion is
+disabled (with '-n' on the command line, or the PrintConv option in the
+API).
 },
     XMP => q{
 XMP stands for "Extensible Metadata Platform", an XML/RDF-based metadata
 format which is being pushed by Adobe.  Information in this format can be
 embedded in many different image file types including JPG, JP2, TIFF, PNG,
-MIFF, PS, PDF, PSD and DNG.
+MIFF, PS, PDF, PSD and DNG, as well as audio file formats supporting ID3v2
+information.
 
 The XMP B<Tag ID>'s aren't listed because in most cases they are identical
 to the B<Tag Name>.
@@ -152,8 +162,16 @@ specifies the information format:  C<integer> is a string of digits
 C<rational> is two C<integer> strings separated by a '/' character, C<date>
 is a date/time string in the format "YYYY:MM:DD HH:MM:SS[+/-HH:MM]",
 C<boolean> is either "True" or "False", and C<lang-alt> is a list of string
-alternatives in different languages. Currently, ExifTool only writes the
-"x-default" language in C<lang-alt> lists.
+alternatives in different languages.
+
+Individual languages for C<lang-alt> tags are accessed by suffixing the tag
+name with a '-', followed by an RFC 3066 language code (ie. "XMP:Title-fr",
+or "Rights-en-US").  A C<lang-alt> tag with no language code accesses the
+"x-default" language, but causes other languages to be deleted when writing.
+The "x-default" language code may be specified when writing a new value to
+write only the default language, but note that all languages are still
+deleted if "x-default" tag is deleted.  When reading, "x-default" is not
+specified.
 
 The XMP tags are organized according to schema B<Namespace> in the following
 tables.  Note that a few of the longer namespace prefixes given below have
@@ -251,8 +269,8 @@ to extract meta information, but they are only a small fraction of the total
 number of available PDF tags.
 },
     DNG => q{
-The main DNG tags are found in the EXIF table.  The tables below define tags
-found within structures of these main tag values.
+The main DNG tags are found in the EXIF table.  The tables below define only
+information found within structures of these main DNG tag values.
 },
     MPEG => q{
 The MPEG format doesn't specify any file-level meta information.  In lieu of
@@ -266,7 +284,9 @@ and RPM).
 },
     Extra => q{
 The extra tags represent information found in the image but not associated
-with any other tag group.
+with any other tag group.  The three writable "pseudo" tags (Filename,
+Directory and FileModifyDate) may be written without the need to rewrite the
+file since their values are not contained within the file data.
 },
     Composite => q{
 The values of the composite tags are derived from the values of other tags.
@@ -291,7 +311,7 @@ L<Image::ExifTool::BuildTagLookup|Image::ExifTool::BuildTagLookup>.
 
 ~head1 AUTHOR
 
-Copyright 2003-2006, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2007, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
@@ -314,7 +334,7 @@ sub new
     my $that = shift;
     my $class = ref($that) || $that || 'Image::ExifTool::BuildTagLookup';
     my $self = bless {}, $class;
-    my %subdirs;
+    my (%subdirs, %isShortcut);
     my %count = (
         'unique tag names' => 0,
         'total tags' => 0,
@@ -323,7 +343,7 @@ sub new
 # loop through all tables, accumulating TagLookup and TagName information
 #
     my (%tagNameInfo, %id, %longID, %longName, %shortName, %tableNum,
-        %tagLookup, %tagExists, %tableWritable, %sepTable);
+        %tagLookup, %tagExists, %tableWritable, %sepTable, %compositeModules);
     $self->{TAG_NAME_INFO} = \%tagNameInfo;
     $self->{TAG_ID} = \%id;
     $self->{LONG_ID} = \%longID;
@@ -334,6 +354,8 @@ sub new
     $self->{TAG_EXISTS} = \%tagExists;
     $self->{TABLE_WRITABLE} = \%tableWritable;
     $self->{SEPARATE_TABLE} = \%sepTable;
+    $self->{COMPOSITE_MODULES} = \%compositeModules;
+    $self->{COUNT} = \%count;
 
     Image::ExifTool::LoadAllTables();
     my @tableNames = sort keys %allTables;
@@ -351,6 +373,14 @@ sub new
         $shortName{$tableName} = $short;    # remember short name
         $tableNum{$tableName} = $tableNum++;
     }
+    # make lookup table to check for shortcut tags
+    my $tag;
+    foreach $tag (keys %Image::ExifTool::Shortcuts::Main) {
+        my $entry = $Image::ExifTool::Shortcuts::Main{$tag};
+        # ignore if shortcut tag name includes itself
+        next if ref $entry eq 'ARRAY' and grep /^$tag$/, @$entry;
+        $isShortcut{lc($tag)} = 1;
+    }
     foreach $tableName (@tableNames) {
         # create short table name
         my $short = $shortName{$tableName};
@@ -370,13 +400,18 @@ sub new
         # save all tag names
         my ($tagID, $binaryTable, $noID, $isIPTC);
         $isIPTC = 1 if $$table{WRITE_PROC} and $$table{WRITE_PROC} eq \&Image::ExifTool::IPTC::WriteIPTC;
-        $noID = 1 if $short =~ /^(Composite|XMP|Extra|Shortcuts|ASF.*)$/;
-        if ($table->{PROCESS_PROC} and
-           ($table->{PROCESS_PROC} eq \&Image::ExifTool::ProcessBinaryData or
-            $table->{PROCESS_PROC} eq \&Image::ExifTool::Real::ProcessSerialData))
+        $noID = 1 if $short =~ /^(Composite|XMP|Extra|Shortcuts|ASF.*)$/ or $table->{VARS}->{NO_ID};
+        if (($table->{VARS} and $table->{VARS}->{INDEX}) or
+            ($table->{PROCESS_PROC} and
+             $table->{PROCESS_PROC} eq \&Image::ExifTool::ProcessBinaryData))
         {
             $binaryTable = 1;
             $id{$tableName} = 'Index';
+        } elsif ($table->{PROCESS_PROC} and
+            $table->{PROCESS_PROC} eq \&Image::ExifTool::Canon::ProcessSerialData)
+        {
+            $binaryTable = 1;
+            $id{$tableName} = 'Sequence';
         } elsif ($isIPTC and $$table{PROCESS_PROC}) { #only the main IPTC table has a PROCESS_PROC
             $id{$tableName} = 'Record';
         } elsif (not $noID) {
@@ -446,39 +481,58 @@ TagID:  foreach $tagID (@keys) {
                     }
                 }
                 my $printConv = $$tagInfo{PrintConv};
-                if (ref $printConv eq 'HASH') {
-                    if ($$tagInfo{SeparateTable}) {
-                        $subdir = 1;
-                        my $s = $$tagInfo{SeparateTable};
-                        ($s = $short) =~ s/ .*// if $s eq '1';
-                        $s .= " $$tagInfo{Name}";
-                        push @values, $s;
-                        $sepTable{$s} = $printConv;
-                    } else {
-                        $caseInsensitive = 0;
-                        my @pk = sort NumbersFirst keys %$printConv;
-                        my $bits;
-                        foreach (@pk) {
-                            next if $_ eq '';
-                            $_ eq 'BITMASK' and $bits = $$printConv{$_}, next;
-                            my $index;
-                            if ($$tagInfo{PrintHex}) {
-                                $index = sprintf('0x%x',$_);
-                            } elsif (/^[-+]?\d+$/) {
-                                $index = $_;
-                            } else {
-                                # ignore unprintable values
-                                next if /[\x00-\x1f\x80-\xff]/;
-                                $index = "'$_'";
-                            }
-                            push @values, "$index = " . $$printConv{$_};
+                if (ref($printConv) =~ /^(HASH|ARRAY)$/) {
+                    my (@printConvList, @indexList, $index);
+                    if (ref $printConv eq 'ARRAY') {
+                        for ($index=0; $index<@$printConv; ++$index) {
+                            next if ref $$printConv[$index] ne 'HASH';
+                            push @printConvList, $$printConv[$index];
+                            push @indexList, $index;
                         }
-                        if ($bits) {
-                            my @pk = sort NumbersFirst keys %$bits;
+                        $printConv = shift @printConvList;
+                        $index = shift @indexList;
+                    }
+                    while (defined $printConv) {
+                        push @values, "[Value $index]" if defined $index;
+                        if ($$tagInfo{SeparateTable}) {
+                            $subdir = 1;
+                            my $s = $$tagInfo{SeparateTable};
+                            $s = $$tagInfo{Name} if $s eq '1';
+                            # add module name if not specified
+                            $s =~ / / or ($short =~ /^(\w+)/ and $s = "$1 $s");
+                            push @values, $s;
+                            $sepTable{$s} = $printConv;
+                            # add PrintHex flag to PrintConv so we can check it later
+                            $$printConv{PrintHex} = 1 if $$tagInfo{PrintHex};
+                        } else {
+                            $caseInsensitive = 0;
+                            my @pk = sort NumbersFirst keys %$printConv;
+                            my $bits;
                             foreach (@pk) {
-                                push @values, "Bit $_ = " . $$bits{$_};
+                                next if $_ eq '';
+                                $_ eq 'BITMASK' and $bits = $$printConv{$_}, next;
+                                my $index;
+                                if ($$tagInfo{PrintHex}) {
+                                    $index = sprintf('0x%x',$_);
+                                } elsif (/^[-+]?\d+$/) {
+                                    $index = $_;
+                                } else {
+                                    # ignore unprintable values
+                                    next if /[\x00-\x1f\x80-\xff]/;
+                                    $index = "'$_'";
+                                }
+                                push @values, "$index = " . $$printConv{$_};
+                            }
+                            if ($bits) {
+                                my @pk = sort NumbersFirst keys %$bits;
+                                foreach (@pk) {
+                                    push @values, "Bit $_ = " . $$bits{$_};
+                                }
                             }
                         }
+                        last unless @printConvList;
+                        $printConv = shift @printConvList;
+                        $index = shift @indexList;
                     }
                 } elsif ($printConv and $printConv =~ /DecodeBits\(\$val,\s*(\{.*\})\s*\)/s) {
                     my $bits = eval $1;
@@ -491,11 +545,9 @@ TagID:  foreach $tagID (@keys) {
                         }
                     }
                 }
-                if ($subdir) {
-                    # flag a subdirectory by setting writable to '-'
-                    # (if this is a writable subdirectory, prefix writable by '-')
+                if ($subdir and not $$tagInfo{SeparateTable}) {
+                    # subdirectories are only writable if specified explicitly
                     $writable = '-' . ($$tagInfo{Writable} ? $writable : '');
-                    $writable = '-N' if $$tagInfo{SeparateTable} and $writable eq '-';
                 } else {
                     # not writable if we can't do the inverse conversions
                     my $noPrintConvInv;
@@ -503,7 +555,8 @@ TagID:  foreach $tagID (@keys) {
                         foreach ('PrintConv','ValueConv') {
                             next unless $$tagInfo{$_};
                             next if $$tagInfo{$_ . 'Inv'};
-                            next if ref $$tagInfo{$_} eq 'HASH';
+                            next if ref($$tagInfo{$_}) =~ /^(HASH|ARRAY)$/;
+                            next if $$tagInfo{WriteAlso};
                             if ($_ eq 'ValueConv') {
                                 undef $writable;
                             } else {
@@ -539,10 +592,18 @@ TagID:  foreach $tagID (@keys) {
                         $writable .= '/' if $$tagInfo{Avoid};
                     }
                     $writable .= '+' if $$tagInfo{List};
+                    # separate tables link like subdirectories (flagged with leading '-')
+                    $writable = "-$writable" if $subdir;
                 }
                 # don't duplicate a tag name unless an entry is different
                 my $name = $$tagInfo{Name};
                 my $lcName = lc($name);
+                # check for conflicts with shortcut names
+                if ($isShortcut{$lcName} and $short ne 'Shortcuts' and
+                    ($$tagInfo{Writable} or not $$tagInfo{SubDirectory}))
+                {
+                    warn "WARNING: $short $name is a shortcut tag!\n";
+                }
                 $name .= '?' if $$tagInfo{Unknown};
                 unless (@tagNames and $tagNames[-1] eq $name and
                     $writeGroup[-1] eq $writeGroup and $writable[-1] eq $writable)
@@ -589,13 +650,16 @@ TagID:  foreach $tagID (@keys) {
                 }
                 $tableWritable{$tableName} = 1;
                 $tagLookup{$lcName}->{$tableNum} = $tagIDs;
+                if ($short eq 'Composite' and $$tagInfo{Module}) {
+                    $compositeModules{$lcName} = $$tagInfo{Module};
+                }
             }
 #
 # save TagName information
 #
             my $tagIDstr;
             if ($tagID =~ /^\d+$/) {
-                if ($binaryTable or $isIPTC or $short =~ /^CanonCustom/) {
+                if ($binaryTable or $isIPTC or ($short =~ /^CanonCustom/ and $tagID < 256)) {
                     $tagIDstr = $tagID;
                 } else {
                     $tagIDstr = sprintf("0x%.4x",$tagID);
@@ -621,7 +685,6 @@ TagID:  foreach $tagID (@keys) {
             push @$info, [ $tagIDstr, \@tagNames, \@writable, \@values, \@require, \@writeGroup ];
         }
     }
-    $self->{COUNT} = \%count;
     return $self;
 }
 
@@ -657,6 +720,7 @@ sub WriteTagLookup($$)
     }
     print OUTFILE "\n# list of tables containing writable tags\n";
     print OUTFILE "my \@tableList = (\n";
+
 #
 # write table list
 #
@@ -723,6 +787,15 @@ sub WriteTagLookup($$)
         next if $$tagLookup{$tag};
         print OUTFILE "\t'$tag' => 1,\n";
     }
+#
+# write module lookup for writable composite tags
+#
+    my $compositeModules = $self->{COMPOSITE_MODULES};
+    print OUTFILE ");\n\n# module names for writable Composite tags\n";
+    print OUTFILE "my \%compositeModules = (\n";
+    foreach (sort keys %$compositeModules) {
+        print OUTFILE "\t'$_' => '$$compositeModules{$_}',\n";
+    }
     print OUTFILE ");\n\n";
 #
 # finish writing TagLookup.pm and clean up
@@ -788,9 +861,10 @@ sub Doc2Pod($)
 sub Doc2Html($)
 {
     my $doc = EscapeHTML(shift);
-    $doc =~ s/\n\n/\n\n<p>/g;
+    $doc =~ s/\n\n/<\/p>\n\n<p>/g;
     $doc =~ s/B&lt;(.*?)&gt;/<b>$1<\/b>/sg;
     $doc =~ s/C&lt;(.*?)&gt;/<code>$1<\/code>/sg;
+    $doc =~ s/L&lt;(.*?)&gt;/<a href="$1">$1<\/a>/sg;
     return $doc;
 }
 
@@ -871,6 +945,7 @@ sub GetTableOrder()
         GeoTiff => 'GPS',
         Leaf    => 'Kodak',
         Unknown => 'Sony',
+        DNG     => 'Unknown',
         PrintIM => 'ICC_Profile',
         Olympus => 'NikonCapture',
         Pentax  => 'Panasonic',
@@ -908,6 +983,7 @@ sub OpenHtmlFile($;$$)
 {
     my ($htmldir, $category, $sepTable) = @_;
     my ($htmlFile, $head, $title, $url, $class);
+    my $top = '';
 
     if ($category) {
         my @names = split ' ', $category;
@@ -925,16 +1001,19 @@ sub OpenHtmlFile($;$$)
         open(HTMLFILE,">>${htmlFile}_tmp") or return 0;
     } else {
         open(HTMLFILE,">${htmlFile}_tmp") or return 0;
-        print HTMLFILE "<html>\n<head>\n<title>$title</title>\n</head>\n";
-        print HTMLFILE "<body text='#000000' $bgBody>\n";
+        print HTMLFILE "$docType<html>\n<head>\n<title>$title</title>\n";
+        print HTMLFILE "<link rel=stylesheet type='text/css' href='style.css' title='Style'>\n";
+        print HTMLFILE "</head>\n<body>\n";
         if ($category ne $class and $docs{$class}) {
-            print HTMLFILE "<h2>$class Tags</h2>\n" or return 0;
-            print HTMLFILE Doc2Html($docs{$class}),"\n" or return 0;
+            print HTMLFILE "<h2 class=top>$class Tags</h2>\n" or return 0;
+            print HTMLFILE '<p>',Doc2Html($docs{$class}),"</p>\n" or return 0;
+        } else {
+            $top = " class=top";
         }
     }
     $head = "<a name='$url'>$head</a>" if $url;
-    print HTMLFILE "<h2>$head</h2>\n" or return 0;
-    print HTMLFILE Doc2Html($docs{$category}),"\n" if $docs{$category};
+    print HTMLFILE "<h2$top>$head</h2>\n" or return 0;
+    print HTMLFILE '<p>',Doc2Html($docs{$category}),"</p>\n" if $docs{$category};
     $createdFiles{$htmlFile} = 1;
     return 1;
 }
@@ -942,8 +1021,11 @@ sub OpenHtmlFile($;$$)
 #------------------------------------------------------------------------------
 # Close all html files and write trailers
 # Returns: true on success
-sub CloseHtmlFiles()
+# Inputs: 0) BuildTagLookup object reference
+sub CloseHtmlFiles($)
 {
+    my $self = shift;
+    my $preserveDate = $$self{PRESERVE_DATE};
     my $success = 1;
     # get the date
     my ($sec,$min,$hr,$day,$mon,$yr) = localtime;
@@ -951,18 +1033,27 @@ sub CloseHtmlFiles()
     $yr += 1900;
     my $date = "$month[$mon] $day, $yr";
     my $htmlFile;
+    my $countNewFiles = 0;
+    my $countSameFiles = 0;
     foreach $htmlFile (keys %createdFiles) {
         my $tmpFile = $htmlFile . '_tmp';
+        my $fileDate = $date;
+        if ($preserveDate) {
+            my @lines = `grep '<i>Last revised' $htmlFile`;
+            $fileDate = $1 if @lines and $lines[-1] =~ m{<i>Last revised (.*)</i>};
+        }
         open(HTMLFILE,">>$tmpFile") or $success = 0, next;
         # write the trailers
-        if ($htmlFile =~ /index\.html$/) {
-            print HTMLFILE "<p><a href='../index.html'>&lt;-- Back to ExifTool Home Page</a>\n";
-        } else {
-            print HTMLFILE "<p><a href='index.html'>&lt;-- ExifTool Tag Names</a>\n"
-        }
         print HTMLFILE "<hr>\n";
         print HTMLFILE "(This document generated automatically by Image::ExifTool::BuildTagLookup)\n";
-        print HTMLFILE "<br><i>Last revised $date</i>\n</body>\n</html>\n" or $success = 0;
+        print HTMLFILE "<br><i>Last revised $fileDate</i>\n";
+        print HTMLFILE "<p class=lf><a href=";
+        if ($htmlFile =~ /index\.html$/) {
+            print HTMLFILE "'../index.html'>&lt;-- Back to ExifTool home page</a></p>\n";
+        } else {
+            print HTMLFILE "'index.html'>&lt;-- ExifTool Tag Names</a></p>\n"
+        }
+        print HTMLFILE "</body>\n</html>\n" or $success = 0;
         close HTMLFILE or $success = 0;
         # check for differences and only use new file if it was changed
         # (so the date only gets updated if changes were really made)
@@ -987,13 +1078,18 @@ sub CloseHtmlFiles()
             }
             close TEMPFILE;
             if ($useNewFile) {
+                ++$countNewFiles;
                 rename $tmpFile, $htmlFile or warn("Error renaming temporary file\n"), $success = 0;
             } else {
+                ++$countSameFiles;
                 unlink $tmpFile;   # erase new file and use existing file
             }
         }
         last unless $success;
     }
+    # save number of files processed so we can check the results later
+    $self->{COUNT}->{'HTML files changed'} = $countNewFiles;
+    $self->{COUNT}->{'HTML files unchanged'} = $countSameFiles;
     return $success;
 }
 
@@ -1013,7 +1109,7 @@ sub WriteTagNames($$)
     my $sepTable = $self->{SEPARATE_TABLE};
     my $success = 1;
     my %htmlFiles;
-    my $columns = 5;    # number of columns in html index
+    my $columns = 6;    # number of columns in html index
     my $percent = int(100 / $columns);
 
     # open the file and write the header
@@ -1022,10 +1118,10 @@ sub WriteTagNames($$)
     mkdir "$htmldir/TagNames";
     OpenHtmlFile($htmldir) or return 0;
     print HTMLFILE "<blockquote>\n";
-    print HTMLFILE "<table width='100%' $bgHeading cellspacing=2 cellpadding=0>\n";
-    print HTMLFILE "<tr $bgBody><td><table width='100%' cellspacing=1 cellpadding=2>\n";
-    print HTMLFILE "<tr $bgHeading><th colspan=$columns><font size='+1'>Tag Table Index</font></th></tr>\n";
-    print HTMLFILE "<tr $bgRow valign='top'><td width='$percent%'>\n";
+    print HTMLFILE "<table width='100%' class=frame><tr><td>\n";
+    print HTMLFILE "<table width='100%' class=inner cellspacing=1><tr class=h>\n";
+    print HTMLFILE "<th colspan=$columns><span class=l>Tag Table Index</span></th></tr>\n";
+    print HTMLFILE "<tr class=b><td width='$percent%'>\n";
     # write the index
     my @tableNames = GetTableOrder();
     push @tableNames, 'Image::ExifTool::Shortcuts::Main';   # do Shortcuts last
@@ -1068,7 +1164,7 @@ sub WriteTagNames($$)
         ++$count;
     }
     print HTMLFILE "\n</td></tr></table></td></tr></table></blockquote>\n";
-    print HTMLFILE Doc2Html($docs{ExifTool2}),"\n";
+    print HTMLFILE '<p>',Doc2Html($docs{ExifTool2}),"</p>\n";
     # write all the tag tables
     while (@tableNames or @sepTables) {
         while (@sepTables) {
@@ -1088,13 +1184,56 @@ sub WriteTagNames($$)
             if (OpenHtmlFile($htmldir, $tableName, 1)) {
                 print HTMLFILE Doc2Html($notes), "\n" if $notes;
                 print HTMLFILE "<blockquote>\n";
-                print HTMLFILE "<table $bgHeading cellspacing=2 cellpadding=0><tr $bgBody><td>\n";
-                print HTMLFILE "<table cellspacing=1 cellpadding=0><tr $bgHeading><th>$head</th><th>Value</th></tr>\n";
+                print HTMLFILE "<table class=frame><tr><td>\n";
+                print HTMLFILE "<table class='inner sep' cellspacing=1>\n";
+                my $align = ' class=r';
+                my $wid = 0;
+                my @keys;
                 foreach (sort NumbersFirst keys %$printConv) {
-                    next if $_ eq 'Notes';
-                    my $val = EscapeHTML($_);
-                    $val =~ /\s/ and $val =  "'$val'";
-                    print HTMLFILE "<tr><td>$val</td><td>= $$printConv{$_}</td></tr>\n";
+                    next if /^(Notes|PrintHex)$/;
+                    $align = '' if $align and /[^\d]/;
+                    my $w = length($_) + length($$printConv{$_});
+                    $wid = $w if $wid < $w;
+                    push @keys, $_;
+                }
+                $wid = length($tableName)+7 if $wid < length($tableName)+7;
+                # print in multiple columns if there is room
+                my $cols = int(80 / ($wid + 4));
+                $cols = 1 if $cols < 1 or $cols > @keys;
+                my $rows = int((scalar(@keys) + $cols - 1) / $cols);
+                my ($r, $c);
+                print HTMLFILE '<tr class=h>';
+                for ($c=0; $c<$cols; ++$c) {
+                    print HTMLFILE "<th>Value</th><th>$head</th>";
+                }
+                print HTMLFILE "</tr>\n";
+                for ($r=0; $r<$rows; ++$r) {
+                    print HTMLFILE '<tr>';
+                    for ($c=0; $c<$cols; ++$c) {
+                        my $key = $keys[$r + $c*$rows];
+                        my ($index, $prt);
+                        if (defined $key) {
+                            $index = $key;
+                            $prt = "= $$printConv{$key}";
+                            if ($$printConv{PrintHex}) {
+                                $index = sprintf('0x%x',$index);
+                            } elsif ($index !~ /^[-+]?\d+$/) {
+                                $index = "'" . EscapeHTML($index) . "'";
+                            }
+                        } else {
+                            $index = $prt = '&nbsp;';
+                        }
+                        my ($ic, $pc);
+                        if ($c & 0x01) {
+                            $pc = ' class=b';
+                            $ic = $align ? " class='r b'" : $pc;
+                        } else {
+                            $ic = $align;
+                            $pc = '';
+                        }
+                        print HTMLFILE "<td$ic>$index</td><td$pc>$prt</td>\n";
+                    }
+                    print HTMLFILE '</tr>';
                 }
                 print HTMLFILE "</table></td></tr></table></blockquote>\n\n";
             }
@@ -1155,7 +1294,7 @@ sub WriteTagNames($$)
                 $wTag = $longTag;
                 warn "Notice: Long tags in $tableName table\n";
             }
-        } elsif ($short =~ /^(Extra|XMP|ASF)/) {
+        } elsif ($short !~ /^(Composite|Shortcuts)/) {
             $wTag += 9;
             $hid = '';
         } else {
@@ -1185,20 +1324,20 @@ sub WriteTagNames($$)
         $line .= sprintf " %-${wGrp}s", 'Group' if $showGrp;
         $line .= ' Writable';
         print PODFILE $line;
+        $line =~ s/Sequence /Sequence\t/;   # don't want an underline after 'Sequence'
         $line =~ s/\S/-/g;
         $line =~ s/- -/---/g;
+        $line =~ tr/\t/ /;
         print PODFILE $line,"\n";
         close HTMLFILE;
         OpenHtmlFile($htmldir, $short) or $success = 0;
-        if ($notes) {
-            print HTMLFILE "<p>" if $docs{$short};
-            print HTMLFILE Doc2Html($notes), "\n";
-        }
+        print HTMLFILE '<p>',Doc2Html($notes), "</p>\n" if $notes;
         print HTMLFILE "<blockquote>\n";
-        print HTMLFILE "<table $bgHeading cellspacing=2 cellpadding=0><tr $bgBody><td>\n";
-        print HTMLFILE "<table cellspacing=1 cellpadding=2><tr $bgHeading>$hid<th>$tagNameHeading</th>\n";
-        print HTMLFILE "<th>Writable</th>$derived<th>Values / ${noteFont}Notes</font></th></tr>\n";
-        my $rowCol = 1;
+        print HTMLFILE "<table class=frame><tr><td>\n";
+        print HTMLFILE "<table class=inner cellspacing=1>\n";
+        print HTMLFILE "<tr class=h>$hid<th>$tagNameHeading</th>\n";
+        print HTMLFILE "<th>Writable</th>$derived<th>Values / ${noteFont}Notes</span></th></tr>\n";
+        my $rowClass = 1;
         my $infoCount = 0;
         my $infoList;
         foreach $infoList (@$info) {
@@ -1210,7 +1349,7 @@ sub WriteTagNames($$)
             } elsif ($tagIDstr =~ /^\d+$/) {
                 $w = $wID - 3;
                 $idStr = sprintf "  %${w}d    ", $tagIDstr;
-                $align = " align='right'";
+                $align = " class=r";
             } else {
                 $tagIDstr =~ s/^'$prefix/'/ if $prefix;
                 $w = $wID;
@@ -1290,7 +1429,7 @@ sub WriteTagNames($$)
             foreach (@$tagNames) {
                 push @htmlTags, EscapeHTML($_);
             }
-            $rowCol = $rowCol ? '' : " $bgRow";
+            $rowClass = $rowClass ? '' : " class=b";
             my $isSubdir;
             if ($$writable[0] =~ /^-/) {
                 $isSubdir = 1;
@@ -1298,12 +1437,12 @@ sub WriteTagNames($$)
                     s/^-(.+)/$1/;
                 }
             }
-            print HTMLFILE "<tr$rowCol valign='top'>\n";
+            print HTMLFILE "<tr$rowClass>\n";
             print HTMLFILE "<td$align>$tagIDstr</td>\n" if $id;
             print HTMLFILE "<td>", join("\n  <br>",@htmlTags), "</td>\n";
-            print HTMLFILE "<td align='center'>",join('<br>',@$writable),"</td>\n";
+            print HTMLFILE "<td class=c>",join('<br>',@$writable),"</td>\n";
             print HTMLFILE '<td>',join("\n  <br>",@$require),"</td>\n" if $composite;
-            print HTMLFILE "<td align='center'>",join('<br>',@$writeGroup),"</td>\n" if $showGrp;
+            print HTMLFILE "<td class=c>",join('<br>',@$writeGroup),"</td>\n" if $showGrp;
             print HTMLFILE "<td>";
             my $close = '';
             my @values;
@@ -1313,7 +1452,7 @@ sub WriteTagNames($$)
                     foreach (@$values) {
                         if (/^\(/) {
                             $smallNote = 1 if $n <= 1;
-                            push @values, ($smallNote ? $noteFontSmall : $noteFont) . "$_</font>";
+                            push @values, ($smallNote ? $noteFontSmall : $noteFont) . "$_</span>";
                             next;
                         }
                         /=/ and push(@values, $_), next;
@@ -1323,7 +1462,7 @@ sub WriteTagNames($$)
                         my $suffix = ' Tags';
                         if ($$sepTable{$_}) {
                             push @sepTables, $_;
-                            $suffix = '';
+                            $suffix = ' Values';
                         }
                         push @values, "--&gt; <a href='$url'>$_$suffix</a>";
                     }
@@ -1332,11 +1471,11 @@ sub WriteTagNames($$)
                 } else {
                     foreach (@$values) {
                         $_ = EscapeHTML($_);
-                        /^\(/ and $_ = "$noteFont$_</font>";
+                        /^\(/ and $_ = "$noteFont$_</span>";
                         push @values, $_;
                     }
-                    print HTMLFILE "<font size='-1'>";
-                    $close = '</font>';
+                    print HTMLFILE "<span class=s>";
+                    $close = '</span>';
                 }
             } else {
                 push @values, '&nbsp;';
@@ -1348,12 +1487,12 @@ sub WriteTagNames($$)
             my $cols = 3;
             ++$cols if $hid;
             ++$cols if $derived;
-            print HTMLFILE "<tr><td colspan=$cols align='center'>[no tags known]</td></tr>\n";
+            print HTMLFILE "<tr><td colspan=$cols class=c>[no tags known]</td></tr>\n";
         }
         print HTMLFILE "</table></td></tr></table></blockquote>\n\n";
     }
     close(HTMLFILE) or $success = 0;
-    CloseHtmlFiles() or $success = 0;
+    CloseHtmlFiles($self) or $success = 0;
     print PODFILE Doc2Pod($docs{PodTrailer}) or $success = 0;
     close(PODFILE) or $success = 0;
     return $success;
@@ -1388,7 +1527,7 @@ documentation.
 
 =head1 AUTHOR
 
-Copyright 2003-2006, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2007, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
