@@ -1,7 +1,7 @@
 #------------------------------------------------------------------------------
 # File:         PDF.pm
 #
-# Description:  Routines for reading PDF files
+# Description:  Read PDF meta information
 #
 # Revisions:    07/11/05 - P. Harvey Created
 #               07/25/05 - P. Harvey Add support for encrypted documents
@@ -17,7 +17,7 @@ use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 require Exporter;
 
-$VERSION = '1.05';
+$VERSION = '1.10';
 
 sub LocateObject($$);
 sub FetchObject($$$$);
@@ -50,11 +50,11 @@ my $cryptInfo;      # encryption object reference (plus additional information)
     GROUPS => { 2 => 'Image' },
     EXTRACT_UNKNOWN => 1, # extract all unknown tags in this directory
     NOTES => q{
-As well as the tags listed below, the PDF specification allows for
-user-defined tags to exist in the Info dictionary.  These tags, which should
-have corresponding XMP-pdfx entries in the PDF Metadata, are also extracted
-by ExifTool.
-},
+        As well as the tags listed below, the PDF specification allows for
+        user-defined tags to exist in the Info dictionary.  These tags, which should
+        have corresponding XMP-pdfx entries in the PDF Metadata, are also extracted
+        by ExifTool.
+    },
     Title       => { },
     Author      => { Groups => { 2 => 'Author' } },
     Subject     => { },
@@ -473,7 +473,7 @@ sub ExtractObject($$;$$)
         Decrypt(\$objData) if $cryptInfo;
         # convert from UTF-16 (big endian) to UTF-8 or Latin if necessary
         if ($objData =~ s/^\xfe\xff//) {
-            $objData = $exifTool->Unicode2Byte($objData, 'MM');
+            $objData = $exifTool->Unicode2Charset($objData, 'MM');
         }
         return $objData;
 #
@@ -644,10 +644,12 @@ sub ReadToNested($;$)
             next unless $2 eq '(' or $2 eq ')';
         } elsif ($2 eq '%') {
             # ignore the comment
-            my $pos = pos($$dataPt);
+            my $pos = pos($$dataPt) - 1;
             # remove everything from '%' up to but not including newline
-            $$dataPt =~ s/%\G.*//;
-            pos($$dataPt) = $pos - 1;
+            $$dataPt =~ /.*/g;
+            my $end = pos($$dataPt);
+            $$dataPt = substr($$dataPt, 0, $pos) . substr($$dataPt, $end);
+            pos($$dataPt) = $pos;
             next;
         }
         if ($closingDelim{$2}) {
@@ -693,13 +695,13 @@ sub DecodeStream($$)
             my $inflate = Compress::Zlib::inflateInit();
             my ($buff, $stat);
             $inflate and ($buff, $stat) = $inflate->inflate($$dict{stream});
-            if ($stat == 1) {
+            if ($inflate and $stat == Compress::Zlib::Z_STREAM_END()) {
                 $$dict{stream} = $buff;
                 # move Filter to prevent double decoding
                 $$dict{oldFilter} = $$dict{Filter};
                 $$dict{Filter} = '';
             } else {
-                $exifTool->Warn("Error inflating stream");
+                $exifTool->Warn('Error inflating stream');
                 return 0;
             }
         } else {
@@ -720,7 +722,7 @@ sub DecodeStream($$)
         my $cols = $dict->{DecodeParms}->{Columns};
         unless ($cols) {
             # currently only support 'up' prediction
-            WarnOnce($exifTool,"No Columns for decoding stream");
+            WarnOnce($exifTool,'No Columns for decoding stream');
             return 0;
         }
         my @bytes = unpack('C*', $$dict{stream});
@@ -896,7 +898,7 @@ sub ProcessDict($$$$;$)
 
     $nesting = ($nesting || 0) + 1;
     if ($nesting > 50) {
-        WarnOnce($exifTool, 'Nesting too deep -- directory ignored');
+        WarnOnce($exifTool, 'Nesting too deep (directory ignored)');
         return;
     }
 #
@@ -979,9 +981,14 @@ sub ProcessDict($$$$;$)
             Image::ExifTool::AddTagToTable($tagTablePtr, $tag, $tagInfo);
         }
         unless ($$tagInfo{SubDirectory}) {
+            # convert from UTF-16 (big endian) to UTF-8 or Latin if necessary
+            # unless this is binary data (hex-encoded strings would not have been converted)
+            if ($val =~ s/^\xfe\xff// and not $$tagInfo{Binary}) {
+                $val = $exifTool->Unicode2Charset($val, 'MM');
+            }
             if ($$tagInfo{List}) {
-                # separate tokens in whitespace delimited lists
-                my @values = split ' ', $val;
+                # separate tokens in comma or whitespace delimited lists
+                my @values = ($val =~ /,/) ? split /,+\s*/, $val : split ' ', $val;
                 foreach $val (@values) {
                     $exifTool->FoundTag($tagInfo, $val);
                 }
@@ -1197,7 +1204,8 @@ sub ReadPDF($$)
     my $dict;
     foreach $dict (@mainDicts) {
         if ($verbose) {
-            printf "PDF dictionary with %d entries:\n", scalar(@{$$dict{tags}});
+            my $n = scalar(@{$$dict{tags}});
+            $exifTool->VPrint(0, "PDF dictionary with $n entries:\n");
         }
         ProcessDict($exifTool, $tagTablePtr, $dict, \%xref);
     }
@@ -1248,7 +1256,7 @@ __END__
 
 =head1 NAME
 
-Image::ExifTool::PDF - Routines for reading PDF files
+Image::ExifTool::PDF - Read PDF meta information
 
 =head1 SYNOPSIS
 
@@ -1263,10 +1271,10 @@ information but only with a limited number of algorithms.
 
 =head1 AUTHOR
 
-Copyright 2003-2005, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2007, Phil Harvey (phil at owl.phy.queensu.ca)
 
-This library is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself.
+This library is free software; you can redistribute it and/or modify it
+under the same terms as Perl itself.
 
 =head1 REFERENCES
 
