@@ -15,7 +15,7 @@ package Image::ExifTool::Shortcuts;
 use strict;
 use vars qw($VERSION);
 
-$VERSION = '1.08';
+$VERSION = '1.16';
 
 # this is a special table used to define command-line shortcuts
 %Image::ExifTool::Shortcuts::Main = (
@@ -138,36 +138,18 @@ $VERSION = '1.08';
         'FlashMode',
         'FlashExposureComp',
     ],
-    # This shortcut intends to remove tags which contain no useful
-    #  information given that the file was created with a Nikon D70.
-    # Use "--d70boring" to suppress tags in this list.
-    D70Boring => [
-        'Make',
-        'XResolution',
-        'YResolution',
-        'ResolutionUnit',
-        'YCbCrPositioning',
-        'FileSource',
-        'SceneType',
-        'ThumbnailOffset',
-        'ThumbnailLength',
-        'CreateDate',
-        'ComponentsConfiguration',
-        'InteropIndex',
-        'InteropVersion',
-        'SubSecTime',
-        'SubSecTimeDigitized',
-        'SensingMethod',
-        'CFAPattern',
-        'ThumbnailImageIFD',
-    ],
     # This shortcut may be useful when copying tags between files to either
     # copy the maker notes as a block or prevent it from being copied
     MakerNotes => [
+        'MakerNotes',   # (for RIFF MakerNotes)
         'MakerNoteCanon',
         'MakerNoteCasio',
         'MakerNoteCasio2',
         'MakerNoteFujiFilm',
+        'MakerNoteHP',
+        'MakerNoteHP2',
+        'MakerNoteHP4',
+        'MakerNoteHP6',
         'MakerNoteJVC',
         'MakerNoteJVCText',
         'MakerNoteKodak1a',
@@ -178,24 +160,30 @@ $VERSION = '1.08';
         'MakerNoteKodak5',
         'MakerNoteKodak6a',
         'MakerNoteKodak6b',
+        'MakerNoteKodak7',
+        'MakerNoteKodakIFD',
         'MakerNoteKodakUnknown',
         'MakerNoteKyocera',
         'MakerNoteMinolta',
         'MakerNoteMinolta2',
         'MakerNoteMinolta3',
-        'MakerNoteMinolta4',
         'MakerNoteNikon',
         'MakerNoteNikon2',
         'MakerNoteNikon3',
         'MakerNoteOlympus',
+        'MakerNoteOlympus2',
         'MakerNoteLeica',
         'MakerNotePanasonic',
         'MakerNotePanasonic2',
         'MakerNotePentax',
+        'MakerNotePentax2',
+        'MakerNotePentax3',
+        'MakerNotePentax4',
         'MakerNoteRicoh',
         'MakerNoteRicohText',
         'MakerNoteSanyo',
-        'MaerkNoteSanyoC4',
+        'MakerNoteSanyoC4',
+        'MakerNoteSanyoPatch',
         'MakerNoteSigma',
         'MakerNoteSony',
         'MakerNoteSonySRF',
@@ -254,7 +242,7 @@ FocalLengthIn35mmFormat.
 
 =head1 AUTHOR
 
-Copyright 2003-2006, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2007, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
